(xbindkey '(mod4 "space")           "freetile tile")
(xbindkey '(mod4 "h")               "freetile focus left")
(xbindkey '(mod4 "j")               "freetile focus down")
(xbindkey '(mod4 "k")               "freetile focus up  ")
(xbindkey '(mod4 "l")               "freetile focus right")
(xbindkey '(mod4 shift "h")         "freetile swap left")
(xbindkey '(mod4 shift "j")         "freetile swap down")
(xbindkey '(mod4 shift "k")         "freetile swap up  ")
(xbindkey '(mod4 shift "l")         "freetile swap right")
(xbindkey '(mod4 shift control "h") "freetile shrink width")
(xbindkey '(mod4 shift control "j") "freetile grow height")
(xbindkey '(mod4 shift control "k") "freetile shrink height")
(xbindkey '(mod4 shift control "l") "freetile grow width")
(xbindkey '(mod4 control "h")       "freetile move left")
(xbindkey '(mod4 control "j")       "freetile move down")
(xbindkey '(mod4 control "k")       "freetile move up  ")
(xbindkey '(mod4 control "l")       "freetile move right")
